/**
 * chroma.js - JavaScript library for color conversions
 *
 * Copyright (c) 2011-2019, Gregor Aisch
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *
 * 1. Redistributions of source code must retain the above copyright notice, this
 * list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright notice,
 * this list of conditions and the following disclaimer in the documentation
 * and/or other materials provided with the distribution.
 *
 * 3. The name Gregor Aisch may not be used to endorse or promote products
 * derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL GREGOR AISCH OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT,
 * INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
 * BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
 * OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 * NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
 * EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * -------------------------------------------------------
 *
 * chroma.js includes colors from colorbrewer2.org, which are released under
 * the following license:
 *
 * Copyright (c) 2002 Cynthia Brewer, Mark Harrower,
 * and The Pennsylvania State University.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing,
 * software distributed under the License is distributed on an
 * "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND,
 * either express or implied. See the License for the specific
 * language governing permissions and limitations under the License.
 *
 * ------------------------------------------------------
 *
 * Named colors are taken from X11 Color Names.
 * http://www.w3.org/TR/css3-color/#svg-color
 *
 * @preserve
 */

!function(t,r){"object"==typeof exports&&"undefined"!=typeof module?module.exports=r():"function"==typeof define&&define.amd?define(r):t.chroma=r()}(this,function(){"use strict";for(var n=function(t,r,n){return void 0===n&&(n=1),t<(r=void 0===r?0:r)?r:n<t?n:t},r={},t=0,e=["Boolean","Number","String","Function","Array","Date","RegExp","Undefined","Null"];t<e.length;t+=1){var o=e[t];r["[object "+o+"]"]=o.toLowerCase()}function a(t){return r[Object.prototype.toString.call(t)]||"object"}var i=Math.PI,u=a,s=function(r,t){return void 0===t&&(t=null),3<=r.length?Array.prototype.slice.call(r):"object"==a(r[0])&&t?t.split("").filter(function(t){return void 0!==r[0][t]}).map(function(t){return r[0][t]}):r[0]},l=function(t){if(t.length<2)return null;var r=t.length-1;return"string"==a(t[r])?t[r].toLowerCase():null},h={format:{},autodetect:[]},c=l,f=function(t){t._clipped=!1,t._unclipped=t.slice(0);for(var r=0;r<=3;r++)r<3?((t[r]<0||255<t[r])&&(t._clipped=!0),t[r]=n(t[r],0,255)):3===r&&(t[r]=n(t[r],0,1));return t},p=u,g=function(){for(var t=[],r=arguments.length;r--;)t[r]=arguments[r];if("object"===p(t[0])&&t[0].constructor&&t[0].constructor===this.constructor)return t[0];var n=!1;if(!(a=c(t))){n=!0,h.sorted||(h.autodetect=h.autodetect.sort(function(t,r){return r.p-t.p}),h.sorted=!0);for(var e=0,o=h.autodetect;e<o.length;e+=1){var a,i=o[e];if(a=i.test.apply(i,t))break}}if(!h.format[a])throw new Error("unknown format: "+t);n=h.format[a].apply(null,n?t:t.slice(0,-1));this._rgb=f(n),3===this._rgb.length&&this._rgb.push(1)};g.prototype.toString=function(){return"function"==p(this.hex)?this.hex():"["+this._rgb.join(",")+"]"};var d=g,b=function(){for(var t=[],r=arguments.length;r--;)t[r]=arguments[r];return new(Function.prototype.bind.apply(b.Color,[null].concat(t)))};b.Color=d,b.version="2.1.1";function v(t){return Math.round(100*t)/100}var i=b,y=s,w=l,m=function(){for(var t=[],r=arguments.length;r--;)t[r]=arguments[r];var n=y(t,"hsla"),e=w(t)||"lsa";return n[0]=v(n[0]||0),n[1]=v(100*n[1])+"%",n[2]=v(100*n[2])+"%","hsla"===e||3<n.length&&n[3]<1?(n[3]=3<n.length?n[3]:1,e="hsla"):n.length=3,e+"("+n.join(",")+")"},_=s,x=function(){for(var t=[],r=arguments.length;r--;)t[r]=arguments[r];var n=(t=_(t,"rgba"))[0],e=t[1],o=t[2];n/=255,e/=255,o/=255;var a,i,u=Math.min(n,e,o),s=Math.max(n,e,o),l=(s+u)/2;return s===u?(a=0,i=Number.NaN):a=l<.5?(s-u)/(s+u):(s-u)/(2-s-u),n==s?i=(e-o)/(s-u):e==s?i=2+(o-n)/(s-u):o==s&&(i=4+(n-e)/(s-u)),(i*=60)<0&&(i+=360),3<t.length&&void 0!==t[3]?[i,a,l,t[3]]:[i,a,l]},M=s,k=l,j=Math.round,F=function(){for(var t=[],r=arguments.length;r--;)t[r]=arguments[r];var n=M(t,"rgba"),e=k(t)||"rgb";return"hsl"==e.substr(0,3)?m(x(n),e):(n[0]=j(n[0]),n[1]=j(n[1]),n[2]=j(n[2]),("rgba"===e||3<n.length&&n[3]<1)&&(n[3]=3<n.length?n[3]:1,e="rgba"),e+"("+n.slice(0,"rgb"===e?3:4).join(",")+")")},N=s,$=Math.round,A=function(){for(var t=[],r=arguments.length;r--;)t[r]=arguments[r];var n,e,o,a=(t=N(t,"hsl"))[0],i=t[1],u=t[2];if(0===i)n=e=o=255*u;else{var s=[0,0,0],l=[0,0,0],h=u<.5?u*(1+i):u+i-u*i,c=2*u-h,a=a/360;s[0]=a+1/3,s[1]=a,s[2]=a-1/3;for(var f=0;f<3;f++)s[f]<0&&(s[f]+=1),1<s[f]&&--s[f],6*s[f]<1?l[f]=c+6*(h-c)*s[f]:2*s[f]<1?l[f]=h:3*s[f]<2?l[f]=c+(h-c)*(2/3-s[f])*6:l[f]=c;n=(a=[$(255*l[0]),$(255*l[1]),$(255*l[2])])[0],e=a[1],o=a[2]}return 3<t.length?[n,e,o,t[3]]:[n,e,o,1]},C=/^rgb\(\s*(-?\d+),\s*(-?\d+)\s*,\s*(-?\d+)\s*\)$/,E=/^rgba\(\s*(-?\d+),\s*(-?\d+)\s*,\s*(-?\d+)\s*,\s*([01]|[01]?\.\d+)\)$/,O=/^rgb\(\s*(-?\d+(?:\.\d+)?)%,\s*(-?\d+(?:\.\d+)?)%\s*,\s*(-?\d+(?:\.\d+)?)%\s*\)$/,S=/^rgba\(\s*(-?\d+(?:\.\d+)?)%,\s*(-?\d+(?:\.\d+)?)%\s*,\s*(-?\d+(?:\.\d+)?)%\s*,\s*([01]|[01]?\.\d+)\)$/,L=/^hsl\(\s*(-?\d+(?:\.\d+)?),\s*(-?\d+(?:\.\d+)?)%\s*,\s*(-?\d+(?:\.\d+)?)%\s*\)$/,I=/^hsla\(\s*(-?\d+(?:\.\d+)?),\s*(-?\d+(?:\.\d+)?)%\s*,\s*(-?\d+(?:\.\d+)?)%\s*,\s*([01]|[01]?\.\d+)\)$/,q=Math.round,g=function(t){var r;if(r=(t=t.toLowerCase().trim()).match(C)){for(var n=r.slice(1,4),e=0;e<3;e++)n[e]=+n[e];return n[3]=1,n}if(r=t.match(E)){for(var o=r.slice(1,5),a=0;a<4;a++)o[a]=+o[a];return o}if(r=t.match(O)){for(var i=r.slice(1,4),u=0;u<3;u++)i[u]=q(2.55*i[u]);return i[3]=1,i}if(r=t.match(S)){for(var s=r.slice(1,5),l=0;l<3;l++)s[l]=q(2.55*s[l]);return s[3]=+s[3],s}if(r=t.match(L)){var h=r.slice(1,4);h[1]*=.01,h[2]*=.01;h=A(h);return h[3]=1,h}if(r=t.match(I)){t=r.slice(1,4);t[1]*=.01,t[2]*=.01;t=A(t);return t[3]=+r[4],t}};g.test=function(t){return C.test(t)||E.test(t)||O.test(t)||S.test(t)||L.test(t)||I.test(t)};var B=g,D=u;d.prototype.css=function(t){return F(this._rgb,t)},i.css=function(){for(var t=[],r=arguments.length;r--;)t[r]=arguments[r];return new(Function.prototype.bind.apply(d,[null].concat(t,["css"])))},h.format.css=B,h.autodetect.push({p:5,test:function(t){for(var r=[],n=arguments.length-1;0<n--;)r[n]=arguments[n+1];if(!r.length&&"string"===D(t)&&B.test(t))return"css"}});var P=s,R=l,U=Math.round,z=function(){for(var t=[],r=arguments.length;r--;)t[r]=arguments[r];var n=P(t,"rgba"),e=n[0],o=n[1],a=n[2],i=n[3],n=R(t)||"auto";void 0===i&&(i=1),"auto"===n&&(n=i<1?"rgba":"rgb");var u=(u="000000"+((e=U(e))<<16|(o=U(o))<<8|(a=U(a))).toString(16)).substr(u.length-6),s=(s="0"+U(255*i).toString(16)).substr(s.length-2);switch(n.toLowerCase()){case"rgba":return"#"+u+s;case"argb":return"#"+s+u;default:return"#"+u}},G=/^#?([A-Fa-f0-9]{6}|[A-Fa-f0-9]{3})$/,H=/^#?([A-Fa-f0-9]{8}|[A-Fa-f0-9]{4})$/,l=function(t){if(t.match(G)){3===(t=4===t.length||7===t.length?t.substr(1):t).length&&(t=(t=t.split(""))[0]+t[0]+t[1]+t[1]+t[2]+t[2]);var r=parseInt(t,16);return[r>>16,r>>8&255,255&r,1]}if(t.match(H)){4===(t=5===t.length||9===t.length?t.substr(1):t).length&&(t=(t=t.split(""))[0]+t[0]+t[1]+t[1]+t[2]+t[2]+t[3]+t[3]);r=parseInt(t,16);return[r>>24&255,r>>16&255,r>>8&255,Math.round((255&r)/255*100)/100]}throw new Error("unknown hex color: "+t)},J=u;d.prototype.hex=function(t){return z(this._rgb,t)},i.hex=function(){for(var t=[],r=arguments.length;r--;)t[r]=arguments[r];return new(Function.prototype.bind.apply(d,[null].concat(t,["hex"])))},h.format.hex=l,h.autodetect.push({p:4,test:function(t){for(var r=[],n=arguments.length-1;0<n--;)r[n]=arguments[n+1];if(!r.length&&"string"===J(t)&&0<=[3,4,5,6,7,8,9].indexOf(t.length))return"hex"}});var K=s,Q=u;d.prototype.hsl=function(){return x(this._rgb)},i.hsl=function(){for(var t=[],r=arguments.length;r--;)t[r]=arguments[r];return new(Function.prototype.bind.apply(d,[null].concat(t,["hsl"])))},h.format.hsl=A,h.autodetect.push({p:2,test:function(){for(var t=[],r=arguments.length;r--;)t[r]=arguments[r];if(t=K(t,"hsl"),"array"===Q(t)&&3===t.length)return"hsl"}});function T(t){return(t/=255)<=.04045?t/12.92:at((t+.055)/1.055,2.4)}function V(t){return et<t?at(t,1/3):t/nt+tt}var W=18,X=.95047,Y=1,Z=1.08883,tt=.137931034,rt=.206896552,nt=.12841855,et=.008856452,ot=s,at=Math.pow,it=function(t,r,n){return t=T(t),r=T(r),n=T(n),[V((.4124564*t+.3575761*r+.1804375*n)/X),V((.2126729*t+.7151522*r+.072175*n)/Y),V((.0193339*t+.119192*r+.9503041*n)/Z)]},ut=function(){for(var t=[],r=arguments.length;r--;)t[r]=arguments[r];var n=ot(t,"rgb"),e=n[0],o=n[1],a=n[2],n=it(e,o,a),e=n[0],o=n[1],a=116*o-16;return[a<0?0:a,500*(e-o),200*(o-n[2])]},st=s,lt=Math.pow,ht=function(t){return 255*(t<=.00304?12.92*t:1.055*lt(t,1/2.4)-.055)},ct=function(t){return rt<t?t*t*t:nt*(t-tt)},l=function(){for(var t=[],r=arguments.length;r--;)t[r]=arguments[r];var n=(t=st(t,"lab"))[0],e=t[1],o=t[2],n=(n+16)/116,e=isNaN(e)?n:n+e/500,o=isNaN(o)?n:n-o/200;return n=Y*ct(n),e=X*ct(e),o=Z*ct(o),[ht(3.2404542*e-1.5371385*n-.4985314*o),ht(-.969266*e+1.8760108*n+.041556*o),ht(.0556434*e-.2040259*n+1.0572252*o),3<t.length?t[3]:1]},ft=s,pt=u;d.prototype.lab=function(){return ut(this._rgb)},i.lab=function(){for(var t=[],r=arguments.length;r--;)t[r]=arguments[r];return new(Function.prototype.bind.apply(d,[null].concat(t,["lab"])))},h.format.lab=l,h.autodetect.push({p:2,test:function(){for(var t=[],r=arguments.length;r--;)t[r]=arguments[r];if(t=ft(t,"lab"),"array"===pt(t)&&3===t.length)return"lab"}});var gt=s,dt=u,bt=Math.round;d.prototype.rgb=function(t){return!1===(t=void 0===t?!0:t)?this._rgb.slice(0,3):this._rgb.slice(0,3).map(bt)},d.prototype.rgba=function(n){return void 0===n&&(n=!0),this._rgb.slice(0,4).map(function(t,r){return!(r<3)||!1===n?t:bt(t)})},i.rgb=function(){for(var t=[],r=arguments.length;r--;)t[r]=arguments[r];return new(Function.prototype.bind.apply(d,[null].concat(t,["rgb"])))},h.format.rgb=function(){for(var t=[],r=arguments.length;r--;)t[r]=arguments[r];var n=gt(t,"rgba");return void 0===n[3]&&(n[3]=1),n},h.autodetect.push({p:3,test:function(){for(var t=[],r=arguments.length;r--;)t[r]=arguments[r];if(t=gt(t,"rgba"),"array"===dt(t)&&(3===t.length||4===t.length&&"number"==dt(t[3])&&0<=t[3]&&t[3]<=1))return"rgb"}});var vt=u;d.prototype.alpha=function(t,r){return void 0===r&&(r=!1),void 0!==t&&"number"===vt(t)?r?(this._rgb[3]=t,this):new d([this._rgb[0],this._rgb[1],this._rgb[2],t],"rgb"):this._rgb[3]},d.prototype.darken=function(t){void 0===t&&(t=1);var r=this.lab();return r[0]-=W*t,new d(r,"lab").alpha(this.alpha(),!0)},d.prototype.brighten=function(t){return this.darken(-(t=void 0===t?1:t))},d.prototype.darker=d.prototype.darken,d.prototype.brighter=d.prototype.brighten,d.prototype.get=function(t){var r=t.split("."),n=r[0],e=r[1],t=this[n]();if(e){r=n.indexOf(e);if(-1<r)return t[r];throw new Error("unknown channel "+e+" in mode "+n)}return t};function yt(t,r,n){void 0===n&&(n=.5);for(var e=[],o=arguments.length-3;0<o--;)e[o]=arguments[o+3];var a=e[0]||"lrgb";if(wt[a]||e.length||(a=Object.keys(wt)[0]),!wt[a])throw new Error("interpolation mode "+a+" is not defined");return"object"!==mt(t)&&(t=new d(t)),"object"!==mt(r)&&(r=new d(r)),wt[a](t,r,n).alpha(t.alpha()+n*(r.alpha()-t.alpha()))}var wt={},mt=u;d.prototype.mix=d.prototype.interpolate=function(t,r){void 0===r&&(r=.5);for(var n=[],e=arguments.length-2;0<e--;)n[e]=arguments[e+2];return yt.apply(void 0,[this,t,r].concat(n))};var _t=u;d.prototype.set=function(t,r,n){void 0===n&&(n=!1);var e=t.split("."),o=e[0],t=e[1],a=this[o]();if(t){var i=o.indexOf(t);if(-1<i){if("string"==_t(r))switch(r.charAt(0)){case"+":case"-":a[i]+=+r;break;case"*":a[i]*=+r.substr(1);break;case"/":a[i]/=+r.substr(1);break;default:a[i]=+r}else{if("number"!==_t(r))throw new Error("unsupported value for Color.set");a[i]=r}e=new d(a,o);return n?(this._rgb=e._rgb,this):e}throw new Error("unknown channel "+t+" in mode "+o)}return a};var xt=Math.sqrt,Mt=Math.pow;wt.lrgb=function(t,r,n){var e=t._rgb,o=e[0],a=e[1],i=e[2],t=r._rgb,e=t[0],r=t[1],t=t[2];return new d(xt(Mt(o,2)*(1-n)+Mt(e,2)*n),xt(Mt(a,2)*(1-n)+Mt(r,2)*n),xt(Mt(i,2)*(1-n)+Mt(t,2)*n),"rgb")};return i.mix=i.interpolate=yt,i.valid=function(){for(var t=[],r=arguments.length;r--;)t[r]=arguments[r];try{return new(Function.prototype.bind.apply(d,[null].concat(t))),!0}catch(t){return!1}},i});
